@@ -24,21 +24,21 @@
 #' @param alpha.weight `logical`; if both the treatment and the covariate are continuous, whether points should be shaded according to their weight. Fainter points are those that have smaller weights. Default is `TRUE`.
 #' 
 #' @section Additional Arguments:
-#' `bal.plot()` works like [bal.tab()] in that it can take a variety of types of inputs and yield the same output for each. Depending on what kind of input is given, different additional parameters are required in `\dots`. For details on what is required and allowed for each additional input and their defaults, see the help file for the [bal.tab()] method associated with the input. The following are the required additional arguments based on each input type:
+#' `bal.plot()` works like [bal.tab()] in that it can take a variety of types of inputs and yield the same output for each. Depending on what kind of input is given, different additional parameters are required in `...`. For details on what is required and allowed for each additional input and their defaults, see the help file for the [bal.tab()] method associated with the input. The following are the required additional arguments based on each input type:
 #'     
-#'         * For `matchit` objects: None
-#'         * For `weightit` objects: None
-#'         * For `ps`, `ps.cont`, `mnps`, and `iptw` objects: (`stop.method`; see [defaults][bal.tab.ps]).
-#'         * For `Match` objects: `formula` and `data` or `covs` and `treat`.
-#'         * For `optmatch` objects: `formula` and `data` or `covs` (`treat` is not required).
-#'         * For `CBPS` objects: None
-#'         * For `ebalance` objects: `formula` and `data` or `covs` and `treat`.
-#'         * For `formula`s: `data`
-#'         * For `data.frame`s: `treat`
-#'         * For `designmatch` objects: `formula` and `data` or `covs` and `treat`.
-#'         * For `sbw` objects: None
-#'         * For `mimids` and `wimids` objects: None, but an argument to `which.imp` should be specified.
-#'         * For other objects processed through `bal.tab()`'s default method, whichever arguments are required to identify treatment, variables, and a conditioning method (if any).
+#' * For `matchit` objects: None
+#' * For `weightit` objects: None
+#' * For `ps`, `ps.cont`, `mnps`, and `iptw` objects: (`stop.method`; see [defaults][bal.tab.ps]).
+#' * For `Match` objects: `formula` and `data` or `covs` and `treat`.
+#' * For `optmatch` objects: `formula` and `data` or `covs` (`treat` is not required).
+#' * For `CBPS` objects: None
+#' * For `ebalance` objects: `formula` and `data` or `covs` and `treat`.
+#' * For `formula`s: `data`
+#' * For `data.frame`s: `treat`
+#' * For `designmatch` objects: `formula` and `data` or `covs` and `treat`.
+#' * For `sbw` objects: None
+#' * For `mimids` and `wimids` objects: None, but an argument to `which.imp` should be specified.
+#' * For other objects processed through `bal.tab()`'s default method, whichever arguments are required to identify treatment, variables, and a conditioning method (if any).
 #'
 #' @returns A `"ggplot"` object, returned invisibly.
 #' 
@@ -106,7 +106,7 @@ bal.plot <- function(x, var.name, ..., which, which.sub = NULL, cluster = NULL,
     
     if (is_null(X$covs.list)) {
         #Point treatment
-        X$covs <- get.C2(X$covs, addl = X$addl, distance = X$distance, cluster = X$cluster, treat = X$treat,
+        X$covs <- .get_C2(X$covs, addl = X$addl, distance = X$distance, cluster = X$cluster, treat = X$treat,
                          drop = FALSE)
         co.names <- attr(X$covs, "co.names")
         if (missing(var.name)) {
@@ -146,7 +146,7 @@ bal.plot <- function(x, var.name, ..., which, which.sub = NULL, cluster = NULL,
     else {
         #Longitudinal
         X$covs.list <- lapply(seq_along(X$covs.list), function(i) {
-            get.C2(X$covs.list[[i]], addl = X$addl.list[[i]], distance = X$distance.list[[i]], cluster = X$cluster,
+            .get_C2(X$covs.list[[i]], addl = X$addl.list[[i]], distance = X$distance.list[[i]], cluster = X$cluster,
                    treat = X$treat.list[[i]], drop = FALSE)
         })
         co.names.list <- lapply(X$covs.list, attr, "co.names")
@@ -214,6 +214,7 @@ bal.plot <- function(x, var.name, ..., which, which.sub = NULL, cluster = NULL,
         else treat.names <- which(appears.in.time)
         
         if (is_not_null(X$weights)) X$weights <- do.call("rbind", lapply(seq_len(sum(appears.in.time)), function(x) X$weights))
+        if (is_not_null(X$s.weights)) X$s.weights <- do.call("c", lapply(seq_len(sum(appears.in.time)), function(x) X$s.weights))
         if (is_not_null(X$cluster)) X$cluster <- rep(X$cluster, sum(appears.in.time))
         if (is_not_null(X$imp)) X$imp <- rep(X$imp, sum(appears.in.time))
     }
@@ -379,12 +380,12 @@ bal.plot <- function(x, var.name, ..., which, which.sub = NULL, cluster = NULL,
                         }
                         else if (any(which.time %in% seq_along(X$covs.list)[appears.in.time])) {
                             .wrn(sprintf("%s does not appear in time period %s",
-                                            add_quotes(var.name), word_list(which.time[!which.time %in% seq_along(X$covs.list)[appears.in.time]], "or"), "."))
+                                            add_quotes(var.name), word_list(which.time[!which.time %in% seq_along(X$covs.list)[appears.in.time]], "or")))
                             which.time <- which.time[which.time %in% seq_along(X$covs.list)[appears.in.time]]
                         }
                         else {
                             .err(sprintf("%s does not appear in time period %s",
-                                         add_quotes(var.name), word_list(which.time, "or"), "."))
+                                         add_quotes(var.name), word_list(which.time, "or")))
                         }
                         in.time <- !is.na(X$time) & X$time %in% which.time
                     }
@@ -399,7 +400,7 @@ bal.plot <- function(x, var.name, ..., which, which.sub = NULL, cluster = NULL,
                         }
                         else if (any(which.time %in% treat.names[appears.in.time])) {
                             time.periods <- word_list(which.time[!which.time %in% treat.names[appears.in.time]], "and")
-                            chk::wrn(sprintf("%s does not appear in the time period%%s corresponding to treatment%%s %s",
+                            .wrn(sprintf("%s does not appear in the time period%%s corresponding to treatment%%s %s",
                                              add_quotes(var.name), time.periods), n = sum(!which.time %in% treat.names[appears.in.time]))
                             which.time <- which.time[which.time %in% treat.names[appears.in.time]]
                         }
@@ -409,7 +410,6 @@ bal.plot <- function(x, var.name, ..., which, which.sub = NULL, cluster = NULL,
                                          add_quotes(var.name), time.periods), n = length(which.time))
                         }
                         in.time <- !is.na(X$time) & treat.names[X$time] %in% which.time
-                        
                     }
                     else {
                         .err(paste0("The following inputs to `which.time` do not correspond to given time periods:\n\t", word_list(which.time[!which.time %in% treat.names])), tidy = FALSE)
@@ -604,10 +604,8 @@ bal.plot <- function(x, var.name, ..., which, which.sub = NULL, cluster = NULL,
                 ggplot2::scale_y_continuous(expand = ggplot2::expansion(mult = c(0, .05)))
         }
         else { #Continuous vars
-            D$var.mean <- ave(D[c("var", "s.weights")], D[facet], 
-                              FUN = function(x) w.m(x[[1]], x[[2]]))[[1]]
-            D$treat.mean <- ave(D[c("treat", "s.weights")], D[facet], 
-                                FUN = function(x) w.m(x[[1]], x[[2]]))[[1]]
+            D$var.mean <- ave_w.m(D[["var"]], D[facet], w = D[["s.weights"]])
+            D$treat.mean <- ave_w.m(D[["treat"]], D[facet], w = D[["s.weights"]])
             
             bp <- ggplot2::ggplot(D, mapping = aes(x = .data$var, y = .data$treat, weight = .data$weights * .data$s.weights))
             
@@ -725,8 +723,7 @@ bal.plot <- function(x, var.name, ..., which, which.sub = NULL, cluster = NULL,
             
             if (type == "histogram") {
                 if (isTRUE(disp.means)) {
-                    D$var.mean <- ave(D[c("var", "weights")], D[c("treat", facet)], 
-                                      FUN = function(x) w.m(x[[1]], x[[2]]))[[1]]
+                    D$var.mean <- ave_w.m(D[["var"]], D[c("treat", facet)], w = D[["weights"]])
                 }
                 
                 if (!is.numeric(args$bins)) args$bins <- 12
@@ -741,7 +738,7 @@ bal.plot <- function(x, var.name, ..., which, which.sub = NULL, cluster = NULL,
                             ggplot2::geom_segment(data = unique(D[D$treat == levels(D$treat)[t], c("var.mean", facet), drop = FALSE]),
                                                   mapping = aes(x = .data$var.mean, xend = .data$var.mean, y = 0, yend = posneg[t]*Inf), 
                                                   color = if (isTRUE(mirror)) "black" else colors[[t]])
-                    return(clear_null(out))
+                    clear_null(out)
                 }
                 ylab <- "Proportion"
                 
@@ -795,8 +792,7 @@ bal.plot <- function(x, var.name, ..., which, which.sub = NULL, cluster = NULL,
                 }
                 
                 if (isTRUE(disp.means)) {
-                    D$var.mean <- ave(D[c("var", "weights")], D[c("treat", facet)], 
-                                      FUN = function(x) w.m(x[[1]], x[[2]]))[[1]]
+                    D$var.mean <- ave_w.m(D[["var"]], D[c("treat", facet)], w = D[["weights"]])
                 }
                 
                 geom_fun <- function(t) {
@@ -812,7 +808,7 @@ bal.plot <- function(x, var.name, ..., which, which.sub = NULL, cluster = NULL,
                             ggplot2::geom_segment(data = unique(D[D$treat == levels(D$treat)[t], c("var.mean", facet), drop = FALSE]),
                                                   mapping = aes(x = .data$var.mean, xend = .data$var.mean, y = 0, yend = posneg[t]*Inf), 
                                                   color = if (isTRUE(mirror)) "black" else colors[[t]])
-                    return(clear_null(out))
+                    clear_null(out)
                 }
                 ylab <- "Density"
                 
@@ -884,7 +880,7 @@ bal.plot <- function(x, var.name, ..., which, which.sub = NULL, cluster = NULL,
         bp <- bp + ggplot2::facet_grid(facet.formula, drop = FALSE, scales = ifelse("subclass" %in% facet, "free_x", "fixed"))
     }
     
-    return(bp)
+   bp
 }
 
 # Helper functions
@@ -913,7 +909,8 @@ compute_density2 <- function(x, w, from, to, bw = "nrd0", adjust = 1,
     nx <- length(x)
     if (is.null(w)) {
         w <- rep(1 / nx, nx)
-    } else {
+    }
+    else {
         w <- w / sum(w)
     }
     
