@@ -97,8 +97,7 @@ STATS[["mean.diffs"]] <- {list(
             }
         }
         
-        xlab <- if (abs) paste("Absolute", xlab.diff) else xlab.diff
-        return(xlab)
+        if (abs) paste("Absolute", xlab.diff) else xlab.diff
     },
     love.plot_add_stars = function(SS.var, variable.names, ...) {
         A <- list(...)
@@ -117,10 +116,10 @@ STATS[["mean.diffs"]] <- {list(
             
             stars <- match_arg(stars, c("none", "std", "raw"))
             if (stars == "none") {
-                warning("Standardized mean differences and raw mean differences are present in the same plot. \nUse the 'stars' argument to distinguish between them and appropriately label the x-axis.", call. = FALSE)
+                .wrn("standardized mean differences and raw mean differences are present in the same plot. \nUse the `stars` argument to distinguish between them and appropriately label the x-axis")
             }
             else {
-                if (!rlang::is_string(star_char)) star_char <- "*"
+                if (!chk::vld_string(star_char)) star_char <- "*"
                 
                 vars_to_star <- setNames(rep(FALSE, length(variable.names)), variable.names)
                 if (stars == "std") {
@@ -137,7 +136,7 @@ STATS[["mean.diffs"]] <- {list(
             }
         }
         
-        return(SS.var)
+        SS.var
     },
     baseline.xintercept = 0,
     threshold.xintercepts = function(threshold, abs) {
@@ -169,7 +168,7 @@ STATS[["variance.ratios"]] <- {list(
         "Variance Ratios"
     },
     love.plot_add_stars = function(SS.var, variable.names, ...) {
-        return(SS.var)
+        SS.var
     },
     baseline.xintercept = 1,
     threshold.xintercepts = function(threshold, abs) {
@@ -204,7 +203,7 @@ STATS[["ks.statistics"]] <- {list(
         "Kolmogorov-Smirnov Statistics"
     },
     love.plot_add_stars = function(SS.var, variable.names, ...) {
-        return(SS.var)
+        SS.var
     },
     baseline.xintercept = 0,
     threshold.xintercepts = function(threshold, abs) {
@@ -233,7 +232,7 @@ STATS[["ovl.coefficients"]] <- {list(
         "Overlapping Coefficients"
     },
     love.plot_add_stars = function(SS.var, variable.names, ...) {
-        return(SS.var)
+        SS.var
     },
     baseline.xintercept = 0,
     threshold.xintercepts = function(threshold, abs) {
@@ -260,12 +259,11 @@ STATS[["correlations"]] <- {list(
     variable_with_the_greatest = "treatment correlation", #also which.stat2 in love.plot
     love.plot_xlab = function(...) {
         A <- list(...)
-        abs <- A$abs
-        if (abs) return("Absolute Treatment-Covariate Correlations")
-        else return("Treatment-Covariate Correlations")
+        if (isTRUE(A$abs)) "Absolute Treatment-Covariate Correlations"
+        else "Treatment-Covariate Correlations"
     },
     love.plot_add_stars = function(SS.var, variable.names, ...) {
-        return(SS.var)
+        SS.var
     },
     baseline.xintercept = 0,
     threshold.xintercepts = function(threshold, abs) {
@@ -295,12 +293,11 @@ STATS[["spearman.correlations"]] <- {list(
     variable_with_the_greatest = "treatment Spearman correlation", #also which.stat2 in love.plot
     love.plot_xlab = function(...) {
         A <- list(...)
-        abs <- A$abs
-        if (abs) return("Absolute Treatment-Covariate Spearman Correlations")
-        else return("Treatment-Covariate Spearman Correlations")
+        if (A$abs) "Absolute Treatment-Covariate Spearman Correlations"
+        else "Treatment-Covariate Spearman Correlations"
     },
     love.plot_add_stars = function(SS.var, variable.names, ...) {
-        return(SS.var)
+        SS.var
     },
     baseline.xintercept = 0,
     threshold.xintercepts = function(threshold, abs) {
@@ -358,8 +355,7 @@ STATS[["mean.diffs.target"]] <- {list(
             }
         }
         
-        xlab <- if (abs) paste("Absolute", xlab.diff) else xlab.diff
-        return(xlab)
+        if (abs) paste("Absolute", xlab.diff) else xlab.diff
     },
     love.plot_add_stars = function(SS.var, variable.names, ...) {
         A <- list(...)
@@ -378,10 +374,10 @@ STATS[["mean.diffs.target"]] <- {list(
             
             stars <- match_arg(stars, c("none", "std", "raw"))
             if (stars == "none") {
-                warning("Standardized mean differences and raw mean differences are present in the same plot. \nUse the 'stars' argument to distinguish between them and appropriately label the x-axis.", call. = FALSE)
+                .wrn("standardized mean differences and raw mean differences are present in the same plot. \nUse the `stars` argument to distinguish between them and appropriately label the x-axis")
             }
             else {
-                if (!rlang::is_string(star_char)) star_char <- "*"
+                if (!chk::vld_string(star_char)) star_char <- "*"
                 
                 vars_to_star <- setNames(rep(FALSE, length(variable.names)), variable.names)
                 if (stars == "std") {
@@ -398,7 +394,7 @@ STATS[["mean.diffs.target"]] <- {list(
             }
         }
         
-        return(SS.var)
+        SS.var
     },
     baseline.xintercept = 0,
     threshold.xintercepts = function(threshold, abs) {
@@ -416,11 +412,11 @@ STATS[["mean.diffs.target"]] <- {list(
         s.d.denom <- "1"
         
         if (is_not_null(weights)) {
-        col_w_smd(C, treat = treat, weights = weights, 
-                  std = std, s.d.denom = s.d.denom,
-                  abs = abs, s.weights = s.weights, bin.vars = bin.vars,
-                  weighted.weights = weighted.weights,
-                  subset = subset)
+            col_w_smd(C, treat = treat, weights = weights, 
+                      std = std, s.d.denom = s.d.denom,
+                      abs = abs, s.weights = s.weights, bin.vars = bin.vars,
+                      weighted.weights = weighted.weights,
+                      subset = subset)
         }
         else rep(NA_real_, ncol(C))
     }
@@ -441,7 +437,7 @@ STATS[["ks.statistics.target"]] <- {list(
         "Target Kolmogorov-Smirnov Statistics"
     },
     love.plot_add_stars = function(SS.var, variable.names, ...) {
-        return(SS.var)
+        SS.var
     },
     baseline.xintercept = 0,
     threshold.xintercepts = function(threshold, abs) {
@@ -464,10 +460,10 @@ STATS[["ks.statistics.target"]] <- {list(
 )}
 
 all_STATS <- function(type) {
-    if (missing(type)) out <- names(STATS)
-    else {
-        type <- match_arg(type, c("bin", "cont"))
-        out <- names(STATS)[get_from_STATS("type") == type]
-    }
+    if (missing(type)) return(unique(names(STATS)))
+    
+    type <- match_arg(type, c("bin", "cont"))
+    out <- names(STATS)[get_from_STATS("type") == type]
+    
     unique(out)
 }
